import functools
from blog import app, db
from blog.models import User, Post, Comment
from blog.form import RegisterForm, LoginForm, PostForm, CommentForm, PostOrderForm
from flask import (
    Blueprint, flash, g, redirect, render_template, request, session, url_for
)
from werkzeug.security import check_password_hash, generate_password_hash


def login_required(view):
    @functools.wraps(view)
    def wrapped_view(**kwargs):
        if g.user is None:
            return redirect(url_for('login'))

        return view(**kwargs)

    return wrapped_view


@app.route("/")
def redir():
    return redirect(url_for('home'))


@app.route("/home", methods=['GET', 'POST'])
def home():
    post_form = PostForm()
    order_form = PostOrderForm(post_order='asc')
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    posts = Post.query.all()
    post_order_form = PostOrderForm(request.form)
    if post_order_form and post_order_form.post_order.data == 'desc':
        posts = [post for post in reversed(posts)]

    return render_template("home.html", posts=posts, username=username, form=post_form, order_form=order_form)


@app.route("/about")
def about():
    return render_template("about.html", title="About")


@app.route("/disclaimer")
def disclaimer():
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    return render_template("disclaimer.html", username=username)


@app.route("/post/<int:post_id>")
def post(post_id):
    post = Post.query.get_or_404(post_id)
    form = CommentForm(post_id=post_id)
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    return render_template("post.html", title=post.title, post=post, comments=post.comment, form=form, username=username)


@login_required
@app.route("/home/add", methods=["POST", "GET"])
def add_post():
    if request.method == 'POST':
        post_form = PostForm()
        current_user = g.user
        app.logger.info(f'{current_user} add post with content {post_form}')
        current_post = Post(title=post_form.title.data, description=post_form.description.data,
                            content=post_form.content.data, author_id=current_user.id)
        db.session.add(current_post)
        db.session.commit()
        flash(u'Posted successfully')
        return redirect(url_for('home'))
    form = PostForm()
    return render_template("add_post.html", username=g.user.username, form=form)


@ login_required
@ app.route("/post/comment/add", methods=["POST"])
def add_comment():
    comment_form = CommentForm()
    current_user = g.user
    app.logger.info(f'{current_user} add comment with content {comment_form}')
    current_comment = Comment(content=comment_form.content.data, author_id=current_user.id,
                              blog_id=comment_form.post_id.data, score=comment_form.score.data, username=current_user.username)
    db.session.add(current_comment)
    db.session.commit()
    flash(u'Comment added successfully')
    return redirect(f'/post/{current_comment.blog_id}')


@ app.route("/login", methods=["GET", "POST"])
def login():
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    login_form = LoginForm()
    if request.method == 'POST':
        if login_form.validate_on_submit():
            login_email = login_form.email.data
            login_password = login_form.password.data
            error = None
            login_user = User.query.filter_by(email=login_email).first()
            if login_user is None:
                error = f'error user info'
            elif not login_user.verify_password(login_password):
                error = f'error user info'
            if error is None:
                session.clear()
                session['username'] = login_user.username
                flash(u'Login successfully')
                return redirect(url_for('home'))
        return redirect(url_for('error'))
    return render_template("login.html", form=login_form, username=username)


@ app.route('/error')
def error():
    return render_template("error.html", username='Guest')


@ app.route('/logout')
def logout():
    session.clear()
    flash(u'You have been logged out.')
    return redirect(url_for('home'))


@ app.route("/register", methods=["GET", "POST"])
def register():
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    form = RegisterForm()
    if request.method == 'POST':
        if form.validate_on_submit():
            try:
                user = User(form.username.data,
                            form.email.data, form.password.data)
                app.logger.info(f'{user} login success')
                db.session.add(user)
                db.session.commit()
                session.clear()
                session['username'] = form.username.data
                flash(u"Your account is created!")
                return redirect(url_for('home'))
            except Exception:
                flash(u'Sorry, there is a problem with your registration')
                return render_template("register.html", form=form, username=username)

    return render_template("register.html", form=form, username=username)


@app.route("/privacy_policy")
def privacy_policy():
    username = 'Guest'
    if session.get('username'):
        username = session.get('username')
    return render_template("policy.html", username=username)


@app.before_request
def load_logged_in_user():
    current_username = session.get('username')
    if current_username is None:
        g.user = None
    else:
        g.user = User.query.filter_by(username=current_username).first()
